/*
 * Name:    mt_init.c
 *
 * Purpose: Initialization functions for the Tcl interface to the MX library.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 1999-2006 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <tcl.h>

#include "mx_util.h"
#include "mx_record.h"
#include "mx_version.h"
#include "mx_handle.h"
#include "mx_plot.h"
#include "mx_tcl.h"
#include "mx_tcl_private.h"

typedef struct {
	char name[80];
	Tcl_ObjCmdProc *fptr;
} MXTCL_COMMAND;

static MXTCL_COMMAND tcl_command[] = {
	{ "mxprivate_init",			Mx_init_cmd },
	{ "mxprivate_get_debug_level",		Mx_get_debug_level_cmd },
	{ "mxprivate_set_debug_level",		Mx_set_debug_level_cmd },
	{ "mxprivate_plotting_is_enabled",	Mx_plotting_is_enabled_cmd },
	{ "mxprivate_set_plot_enable",		Mx_set_plot_enable_cmd },
	{ "mxprivate_set_program_name",		Mx_set_program_name_cmd },

	{ "mxprivate_process_exists",		Mx_process_exists_cmd },

	{ "mxprivate_getch",			Mx_getch_cmd },
	{ "mxprivate_kbhit",			Mx_kbhit_cmd },
	{ "mxprivate_user_requested_interrupt",
					Mx_user_requested_interrupt_cmd },
	{ "mxprivate_set_user_interrupt_function",
					Mx_set_user_interrupt_function_cmd },
	{ "mxprivate_set_scan_pause_request_handler",
					Mx_set_scan_pause_request_handler_cmd },

	{ "mxprivate_info",			Mx_info_cmd },
	{ "mxprivate_set_info_output_function",		
					Mx_set_info_output_function_cmd },
	{ "mxprivate_warning",			Mx_warning_cmd },
	{ "mxprivate_set_warning_output_function",		
					Mx_set_warning_output_function_cmd },
	{ "mxprivate_set_error_output_function",		
					Mx_set_error_output_function_cmd },
	{ "mxprivate_debug",			Mx_debug_cmd },
	{ "mxprivate_set_debug_output_function",		
					Mx_set_debug_output_function_cmd },
	{ "mxprivate_info_dialog",			Mx_info_dialog_cmd },
	{ "mxprivate_set_info_dialog_function",		
					Mx_set_info_dialog_function_cmd },

	{ "mxprivate_create_record_list",	Mx_create_record_list_cmd },
	{ "mxprivate_read_database",		Mx_read_database_cmd },
	{ "mxprivate_finish_database_initialization",
					Mx_finish_database_initialization_cmd },
	{ "mxprivate_initialize_hardware",	Mx_initialize_hardware_cmd },
	{ "mxprivate_create_record_from_description",
					Mx_create_record_from_description_cmd },
	{ "mxprivate_finish_record_initialization",
					Mx_finish_record_initialization_cmd },
	{ "mxprivate_delete_record",		Mx_delete_record_cmd },
	{ "mxprivate_get_record",		Mx_get_record_cmd },

	{ "mxprivate_get_next_record",		Mx_get_next_record_cmd },
	{ "mxprivate_get_previous_record",	Mx_get_previous_record_cmd },
	{ "mxprivate_get_record_name",		Mx_get_record_name_cmd },
	{ "mxprivate_create_description_from_record",
					Mx_create_description_from_record_cmd },
	{ "mxprivate_get_field",		Mx_get_field_cmd },
	{ "mxprivate_set_field",		Mx_set_field_cmd },
	{ "mxprivate_get_object_class_name",	Mx_get_object_class_name_cmd },
	{ "mxprivate_get_version",		Mx_get_version_cmd },
	{ "mxprivate_log_open",			Mx_log_open_cmd },
	{ "mxprivate_log_close",		Mx_log_close_cmd },

	{ "mxprivate_rs232_getchar",		Mx_rs232_getchar_cmd },
	{ "mxprivate_rs232_putchar",		Mx_rs232_putchar_cmd },
	{ "mxprivate_rs232_getline",		Mx_rs232_getline_cmd },
	{ "mxprivate_rs232_putline",		Mx_rs232_putline_cmd },
	{ "mxprivate_rs232_num_input_bytes_available",
				Mx_rs232_num_input_bytes_available_cmd },
	{ "mxprivate_rs232_discard_unread_input",
					Mx_rs232_discard_unread_input_cmd },
	{ "mxprivate_rs232_discard_unwritten_output",
					Mx_rs232_discard_unwritten_output_cmd },

	{ "mxprivate_amplifier_get_gain",	Mx_amplifier_get_gain_cmd },
	{ "mxprivate_amplifier_set_gain",	Mx_amplifier_set_gain_cmd },
	{ "mxprivate_amplifier_get_offset",	Mx_amplifier_get_offset_cmd },
	{ "mxprivate_amplifier_set_offset",	Mx_amplifier_set_offset_cmd },
	{ "mxprivate_amplifier_get_time_constant",
					Mx_amplifier_get_time_constant_cmd },
	{ "mxprivate_amplifier_set_time_constant",
					Mx_amplifier_set_time_constant_cmd },

	{ "mxprivate_analog_input_read",	Mx_analog_input_read_cmd },
	{ "mxprivate_analog_input_read_raw_long",
					Mx_analog_input_read_raw_long_cmd },
	{ "mxprivate_analog_input_read_raw_double",
					Mx_analog_input_read_raw_double_cmd },
	{ "mxprivate_analog_input_get_dark_current",
					Mx_analog_input_get_dark_current_cmd },
	{ "mxprivate_analog_input_set_dark_current",
					Mx_analog_input_set_dark_current_cmd },

	{ "mxprivate_analog_output_read",	Mx_analog_output_read_cmd },
	{ "mxprivate_analog_output_read_raw_long",
					Mx_analog_output_read_raw_long_cmd },
	{ "mxprivate_analog_output_read_raw_double",
					Mx_analog_output_read_raw_double_cmd },
	{ "mxprivate_analog_output_write",	Mx_analog_output_write_cmd },
	{ "mxprivate_analog_output_write_raw_long",
					Mx_analog_output_write_raw_long_cmd },
	{ "mxprivate_analog_output_write_raw_double",
					Mx_analog_output_write_raw_double_cmd },

	{ "mxprivate_digital_input_read",	Mx_digital_input_read_cmd },

	{ "mxprivate_digital_output_read",	Mx_digital_output_read_cmd },
	{ "mxprivate_digital_output_write",	Mx_digital_output_write_cmd },

	{ "mxprivate_mca_start",		Mx_mca_start_cmd },
	{ "mxprivate_mca_stop",			Mx_mca_stop_cmd },
	{ "mxprivate_mca_read",			Mx_mca_read_cmd },
	{ "mxprivate_mca_clear",		Mx_mca_clear_cmd },
	{ "mxprivate_mca_is_busy",		Mx_mca_is_busy_cmd },
	{ "mxprivate_mca_start_without_preset",
					Mx_mca_start_without_preset_cmd },
	{ "mxprivate_mca_start_with_preset",	Mx_mca_start_with_preset_cmd },
	{ "mxprivate_mca_start_for_preset_live_time",
					Mx_mca_start_for_preset_live_time_cmd },
	{ "mxprivate_mca_start_for_preset_real_time",
					Mx_mca_start_for_preset_real_time_cmd },
	{ "mxprivate_mca_start_for_preset_count",
					Mx_mca_start_for_preset_count_cmd },
	{ "mxprivate_mca_get_preset_type",	Mx_mca_get_preset_type_cmd },
	{ "mxprivate_mca_set_preset_type",	Mx_mca_set_preset_type_cmd },
	{ "mxprivate_mca_get_preset_count_region",
					Mx_mca_get_preset_count_region_cmd },
	{ "mxprivate_mca_set_preset_count_region",
					Mx_mca_set_preset_count_region_cmd },
	{ "mxprivate_mca_get_roi",		Mx_mca_get_roi_cmd },
	{ "mxprivate_mca_set_roi",		Mx_mca_set_roi_cmd },
	{ "mxprivate_mca_get_roi_integral",	Mx_mca_get_roi_integral_cmd },
	{ "mxprivate_mca_get_rois",		Mx_mca_get_rois_cmd },
	{ "mxprivate_mca_set_rois",		Mx_mca_set_rois_cmd },
	{ "mxprivate_mca_get_roi_integrals",	Mx_mca_get_roi_integrals_cmd },
	{ "mxprivate_mca_get_num_channels",	Mx_mca_get_num_channels_cmd },
	{ "mxprivate_mca_set_num_channels",	Mx_mca_set_num_channels_cmd },
	{ "mxprivate_mca_get_channel",		Mx_mca_get_channel_cmd },
	{ "mxprivate_mca_get_real_time",	Mx_mca_get_real_time_cmd },
	{ "mxprivate_mca_get_live_time",	Mx_mca_get_live_time_cmd },
	{ "mxprivate_mca_get_counts",		Mx_mca_get_counts_cmd },
	{ "mxprivate_mca_get_soft_roi",		Mx_mca_get_soft_roi_cmd },
	{ "mxprivate_mca_set_soft_roi",		Mx_mca_set_soft_roi_cmd },
	{ "mxprivate_mca_get_soft_integral",	Mx_mca_get_soft_integral_cmd },

	{ "mxprivate_motor_is_busy",		Mx_motor_is_busy_cmd },
	{ "mxprivate_motor_move_absolute",	Mx_motor_move_absolute_cmd },
	{ "mxprivate_motor_get_position",	Mx_motor_get_position_cmd },
	{ "mxprivate_motor_set_position",	Mx_motor_set_position_cmd },
	{ "mxprivate_motor_soft_abort",		Mx_motor_soft_abort_cmd },
	{ "mxprivate_motor_immediate_abort",	Mx_motor_immediate_abort_cmd },
	{ "mxprivate_motor_positive_limit_hit",
					Mx_motor_positive_limit_hit_cmd },
	{ "mxprivate_motor_negative_limit_hit",
					Mx_motor_negative_limit_hit_cmd },
	{ "mxprivate_motor_find_home_position",
					Mx_motor_find_home_position_cmd },
	{ "mxprivate_motor_constant_velocity_move",
					Mx_motor_constant_velocity_move_cmd },
	{ "mxprivate_motor_get_status",		Mx_motor_get_status_cmd },
	{ "mxprivate_motor_get_extended_status",
					Mx_motor_get_extended_status_cmd },
	{ "mxprivate_motor_get_speed",		Mx_motor_get_speed_cmd },
	{ "mxprivate_motor_set_speed",		Mx_motor_set_speed_cmd },
	{ "mxprivate_motor_get_base_speed",	Mx_motor_get_base_speed_cmd },
	{ "mxprivate_motor_set_base_speed",	Mx_motor_set_base_speed_cmd },
	{ "mxprivate_motor_get_maximum_speed",
					Mx_motor_get_maximum_speed_cmd },
	{ "mxprivate_motor_set_maximum_speed",
					Mx_motor_set_maximum_speed_cmd },
	{ "mxprivate_motor_get_acceleration_type",
					Mx_motor_get_acceleration_type_cmd },
	{ "mxprivate_motor_get_raw_acceleration_parameters",
				Mx_motor_get_raw_acceleration_parameters_cmd },
	{ "mxprivate_motor_set_raw_acceleration_parameters",
				Mx_motor_set_raw_acceleration_parameters_cmd },
	{ "mxprivate_motor_home_search_succeeded",
					Mx_motor_home_search_succeeded_cmd },

	{ "mxprivate_relay_command",		Mx_relay_command_cmd },
	{ "mxprivate_get_relay_status",		Mx_get_relay_status_cmd },

	{ "mxprivate_scaler_clear",		Mx_scaler_clear_cmd },
	{ "mxprivate_scaler_overflow_set",	Mx_scaler_overflow_set_cmd },
	{ "mxprivate_scaler_read",		Mx_scaler_read_cmd },
	{ "mxprivate_scaler_read_raw",		Mx_scaler_read_raw_cmd },
	{ "mxprivate_scaler_get_dark_current",
					Mx_scaler_get_dark_current_cmd },
	{ "mxprivate_scaler_set_dark_current",
					Mx_scaler_set_dark_current_cmd },

	{ "mxprivate_timer_is_busy",		Mx_timer_is_busy_cmd },
	{ "mxprivate_timer_start",		Mx_timer_start_cmd },
	{ "mxprivate_timer_stop",		Mx_timer_stop_cmd },

	{ "mxprivate_scan_perform_scan",	Mx_scan_perform_scan_cmd },

	{ "mxprivate_send_variable",		Mx_send_variable_cmd },
	{ "mxprivate_receive_variable",         Mx_receive_variable_cmd },
};

static int num_tcl_commands = sizeof(tcl_command) / sizeof(tcl_command[0]);

/* The following is the one global variable in libMxTcl. */

MX_HANDLE_TABLE *mxtcl_global_record_handle_table;

/*-------------------------------------------------------*/

MX_EXPORT int
Mxtcl_Init( Tcl_Interp *interp )
{
	return Mx_Init( interp );
}

/*-------------------------------------------------------*/

MX_EXPORT int
Mx_Init( Tcl_Interp *interp )
{
	char version[20];
	int i;

	/* Setup the Tcl stubs mechanism. */

	if ( Tcl_InitStubs( interp, "8.3", 0 ) == NULL ) {
		return TCL_ERROR;
	}

	/* Register ordinary Tcl commands. */

	for ( i = 0; i < num_tcl_commands; i++ ) {

		Tcl_CreateObjCommand( interp,
				tcl_command[i].name, tcl_command[i].fptr,
				(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

	}

	/* Declare that we implement part of the Mx package. */

	snprintf(version, sizeof(version), "%d.%d",
		mx_get_major_version(), mx_get_minor_version());

	Tcl_PkgProvide( interp, "Mx", version );

	/* Bring the global environment into the context of the libMxTcl
	 * library.  Using getenv() on any variable name seems to do this
	 * as a side effect.
	 */

	(void) getenv( "XYZZY" );

	/* Initialize Tcl dynamic strings used by the input and
	 * output functions.
	 */

	Mx_user_interrupt_dstring_init();
	Mx_scan_pause_request_dstring_init();
	Mx_output_dstring_init();

	return TCL_OK;
}

MX_EXPORT int
Mx_init_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_init_cmd()";

	mx_status_type mx_status;

	/* Initialize the MX runtime environment. */

	mx_status = mx_initialize_runtime();

	if ( mx_status.code != MXE_SUCCESS ) {
		return mxtcl_error( interp, mx_status );
	}

	MX_DEBUG( 2,("%s invoked", fname));

	/* Also trap ctrl-C interrupts. */

	signal( SIGINT, SIG_IGN );

	/* Initialize the drivers for the various MX record types. */

	mx_status = mx_initialize_drivers();

	if ( mx_status.code != MXE_SUCCESS ) {
		return mxtcl_error( interp, mx_status );
	}

	/* Create the global handle table for the MX record.  Configure the
	 * table to be incremented in blocks of 100 handles.
	 */

	mx_status = mx_create_handle_table( &mxtcl_global_record_handle_table,
						100, 1 );

	if ( mx_status.code != MXE_SUCCESS ) {
		return mxtcl_error( interp, mx_status );
	}

	return TCL_OK;
}

MX_EXPORT int
Mx_get_debug_level_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	int debug_level;

	if ( objc != 1 ) {
		Tcl_WrongNumArgs( interp, 0, objv,
		"Usage: get_debug_level" );
		return TCL_ERROR;
	}

	debug_level = mx_get_debug_level();

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, debug_level );

	return TCL_OK;
}

MX_EXPORT int
Mx_set_debug_level_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_set_debug_level_cmd()";

	int debug_level, tcl_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: set_debug_level debug_level" );
		return TCL_ERROR;
	}

	tcl_status = Tcl_GetIntFromObj( interp, objv[1], &debug_level );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, objv[1], "integer", fname );

	mx_set_debug_level( debug_level );

	return TCL_OK;
}

MX_EXPORT int
Mx_plotting_is_enabled_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	MX_RECORD *record;
	Tcl_Obj *resultPtr;
	int plot_enable_value;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: get_debug_level handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	plot_enable_value = mx_plotting_is_enabled( record );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, plot_enable_value );

	return TCL_OK;
}

MX_EXPORT int
Mx_set_plot_enable_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_set_plot_enable_cmd()";

	MX_RECORD *record;
	int plot_enable_value, tcl_status;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: set_plot_enable handle plot_enable_value" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetIntFromObj( interp, objv[2], &plot_enable_value );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, objv[2], "integer", fname );

	mx_set_plot_enable( record, plot_enable_value );

	return TCL_OK;
}

MX_EXPORT int
Mx_set_program_name_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_set_program_name_cmd()";

	MX_RECORD *record_list;
	char *program_name;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: set_program_name handle program_name" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record_list,
							interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	program_name = Tcl_GetString( objv[2] );

	if ( program_name == NULL )
		return mxtcl_object_parse_error(
				interp, objv[2], "string", fname );

	mx_status = mx_set_program_name( record_list, program_name );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_process_exists_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_process_exists_cmd()";

	Tcl_Obj *resultPtr;
	long process_id;
	int process_status, tcl_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: process_exists process_id" );
		return TCL_ERROR;
	}

	tcl_status = Tcl_GetLongFromObj( interp, objv[1], &process_id );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, objv[1], "long", fname );

	process_status = mx_process_exists( (unsigned long) process_id );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, process_status );

	return TCL_OK;
}

