/*
 * Name:    mt_mca.c
 *
 * Purpose: Tcl module interface to generic MX MCA functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2005 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>

#include <tcl.h>

#include "mx_util.h"
#include "mx_record.h"
#include "mx_mca.h"
#include "mx_handle.h"
#include "mx_array.h"
#include "mx_tcl.h"
#include "mx_tcl_private.h"

MX_EXPORT int
Mx_mca_is_busy_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	int busy;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_is_busy( record, &busy );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, busy );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_start_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	MX_RECORD *record;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_start( record );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_stop_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	MX_RECORD *record;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_stop( record );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_read_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr, *temp_object;
	MX_RECORD *record;
	unsigned long i, num_channels;
	unsigned long *channel_array;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_read( record, &num_channels, &channel_array );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	/* Create a new object to hold the returned channel values. */

	temp_object = Tcl_NewObj();

	resultPtr = Tcl_GetObjResult( interp );

	Tcl_SetListObj( resultPtr, 0, NULL );

	/* Insert the channel values into the object. */

	for ( i = 0; i < num_channels; i++ ) {

		Tcl_SetLongObj( temp_object, channel_array[i] );

		Tcl_ListObjAppendElement( interp, resultPtr, temp_object );
	}

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_clear_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	MX_RECORD *record;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_clear( record );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_start_without_preset_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	MX_RECORD *record;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_start_without_preset( record );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_start_with_preset_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_start_with_preset_cmd()";

	MX_RECORD *record;
	int tcl_status, preset_type;
	double preset_value;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetIntFromObj( interp, objv[2], &preset_type );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "int", fname );

	tcl_status = Tcl_GetDoubleFromObj( interp, objv[3], &preset_value );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[3], "double", fname );

	mx_status = mx_mca_start_with_preset( record,
						preset_type, preset_value );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_start_for_preset_live_time_cmd(ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_start_for_preset_live_time_cmd()";

	MX_RECORD *record;
	int tcl_status;
	double preset_live_time;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetDoubleFromObj( interp, objv[2], &preset_live_time );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "double", fname );

	mx_status = mx_mca_start_for_preset_live_time( record,
							preset_live_time );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_start_for_preset_real_time_cmd(ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_start_for_preset_real_time_cmd()";

	MX_RECORD *record;
	int tcl_status;
	double preset_real_time;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetDoubleFromObj( interp, objv[2], &preset_real_time );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "double", fname );

	mx_status = mx_mca_start_for_preset_real_time( record,
							preset_real_time );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_start_for_preset_count_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_start_for_preset_count_cmd()";

	MX_RECORD *record;
	int tcl_status;
	long preset_count;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &preset_count );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "long", fname );

	mx_status = mx_mca_start_for_preset_count( record,
						(unsigned long) preset_count );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_preset_type_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	int preset_type;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_get_preset_type( record, &preset_type );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, preset_type );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_set_preset_type_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_set_preset_type_cmd()";

	MX_RECORD *record;
	int tcl_status, preset_type;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetIntFromObj( interp, objv[2], &preset_type );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "int", fname );

	mx_status = mx_mca_set_preset_type( record, preset_type );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_preset_count_region_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr, *temp_object;
	MX_RECORD *record;
	unsigned long preset_count_region[2];
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_get_preset_count_region( record,
						preset_count_region );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	/* The preset count region boundaries are returned
	 * as a 2-element list.
	 */

	temp_object = Tcl_NewObj();

	resultPtr = Tcl_GetObjResult( interp );

	Tcl_SetListObj( resultPtr, 0, NULL );

	/* Insert the region boundaries into the list. */

	Tcl_SetLongObj( temp_object, preset_count_region[0] );
	Tcl_ListObjAppendElement( interp, resultPtr, temp_object );

	Tcl_SetLongObj( temp_object, preset_count_region[1] );
	Tcl_ListObjAppendElement( interp, resultPtr, temp_object );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_set_preset_count_region_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_set_preset_count_region_cmd()";

	Tcl_Obj **list_elements;
	MX_RECORD *record;
	int tcl_status, list_length;
	unsigned long preset_count_region[2];
	long lower_boundary, upper_boundary;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	/* The last argument is expected to be a 2-element list containing
	 * the region of interest boundaries.
	 */

	tcl_status = Tcl_ListObjGetElements( interp, objv[2],
						&list_length, &list_elements );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "list", fname );

	/* Copy the preset_count_region boundaries in the list
	 * to the preset_count_region array.
	 */

	tcl_status = Tcl_GetLongFromObj( interp,
				list_elements[0], &lower_boundary );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, list_elements[0], "long", fname );

	tcl_status = Tcl_GetLongFromObj( interp,
				list_elements[1], &upper_boundary );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, list_elements[1], "long", fname );

	preset_count_region[0] = (unsigned long) lower_boundary;
	preset_count_region[1] = (unsigned long) upper_boundary;

	mx_status = mx_mca_set_preset_count_region( record,
						preset_count_region );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_roi_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_get_roi_cmd()";

	Tcl_Obj *resultPtr, *temp_object;
	MX_RECORD *record;
	int tcl_status;
	long roi_number;
	unsigned long roi[2];
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &roi_number );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "long", fname );

	mx_status = mx_mca_get_roi( record, (unsigned long) roi_number, roi );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	/* The region of interest boundaries are returned
	 * as a 2-element list.
	 */

	temp_object = Tcl_NewObj();

	resultPtr = Tcl_GetObjResult( interp );

	Tcl_SetListObj( resultPtr, 0, NULL );

	/* Insert the region of interest boundaries into the list. */

	Tcl_SetLongObj( temp_object, roi[0] );
	Tcl_ListObjAppendElement( interp, resultPtr, temp_object );

	Tcl_SetLongObj( temp_object, roi[1] );
	Tcl_ListObjAppendElement( interp, resultPtr, temp_object );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_set_roi_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_set_roi_cmd()";

	MX_RECORD *record;
	int tcl_status;
	long roi_number;
	unsigned long roi[2];
	long lower_boundary, upper_boundary;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &roi_number );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "long", fname );

	/* Copy the roi boundaries to the roi array. */

	tcl_status = Tcl_GetLongFromObj( interp, objv[3], &lower_boundary );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, objv[3], "long", fname );

	tcl_status = Tcl_GetLongFromObj( interp, objv[4], &upper_boundary );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, objv[4], "long", fname );

	roi[0] = (unsigned long) lower_boundary;
	roi[1] = (unsigned long) upper_boundary;

	mx_status = mx_mca_set_roi( record, (unsigned long) roi_number, roi );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_roi_integral_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_get_roi_integral_cmd()";

	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	int tcl_status;
	long roi_number;
	unsigned long roi_integral;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &roi_number );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "long", fname );

	mx_status = mx_mca_get_roi_integral( record,
						(unsigned long) roi_number,
						&roi_integral );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetLongObj( resultPtr, (long) roi_integral );

	return TCL_OK;
}

/* FIXME - I am not sure that the following routine works. */

MX_EXPORT int
Mx_mca_get_rois_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_get_rois_cmd()";

	Tcl_Obj *resultPtr, *top_row_object, *row_element;
	MX_RECORD *record;
	MX_MCA *mca;
	unsigned long i, current_num_rois;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	/* Find out how many ROIs are currently in use. */

	mx_status = mx_mca_get_pointers( record, &mca, NULL, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	current_num_rois = mca->current_num_rois;

	/* Read the ROI boundaries for all ROIs. */

	mx_status = mx_mca_get_roi_array( record, current_num_rois, NULL );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	/* Create a list to contain the set of ROI boundaries. */

	resultPtr = Tcl_GetObjResult( interp );

	Tcl_SetListObj( resultPtr, 0, NULL );

	top_row_object = Tcl_NewObj();

	row_element = Tcl_NewObj();

	/* Copy the ROI boundaries to the new list. */

	for ( i = 0; i < current_num_rois; i++ ) {

		/* Each element of the top level list is itself a 2-element
		 * list containing the boundaries of one ROI.
		 */

		Tcl_SetListObj( top_row_object, 0, NULL );
		Tcl_ListObjAppendElement( interp, resultPtr, top_row_object );

		/* Append the two ROI boundaries. */

		Tcl_SetLongObj( row_element, (mca->roi_array)[i][0] );
		Tcl_ListObjAppendElement( interp, top_row_object, row_element );

		Tcl_SetLongObj( row_element, (mca->roi_array)[i][1] );
		Tcl_ListObjAppendElement( interp, top_row_object, row_element );
	}

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_set_rois_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_set_rois_cmd()";

	Tcl_Obj **list_elements, *top_row_element;
	MX_RECORD *record;
	MX_MCA *mca;
	int tcl_status, list_length;
	unsigned long i, current_num_rois;
	unsigned long **roi_array;
	long lower_boundary, upper_boundary;
	long dimension_array[2];
	size_t element_size_array[2];
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	/* Find out how many ROIs are currently in use. */

	mx_status = mx_mca_get_pointers( record, &mca, NULL, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	current_num_rois = mca->current_num_rois;

	/* The last argument is expected to be a list of 2-element lists
	 * containing the region of interest boundaries.
	 */

	dimension_array[0] = (long) current_num_rois;
	dimension_array[1] = 2;

	element_size_array[0] = sizeof( unsigned long );
	element_size_array[1] = sizeof( unsigned long * );

	roi_array = (unsigned long **) mx_allocate_array( 2,
					dimension_array, element_size_array );

	if ( roi_array == NULL ) {
		mx_status = mx_error( MXE_OUT_OF_MEMORY, fname,
		"Ran out of memory trying to allocate a %lu by 2 element "
			"ROI array.", current_num_rois );

		return mxtcl_error( interp, mx_status );
	}

	/* Copy the ROI boundaries to roi_array. */

	for ( i = 0; i < current_num_rois; i++ ) {

		tcl_status = Tcl_ListObjIndex( interp, objv[2],
						i, &top_row_element );

		if ( tcl_status != TCL_OK )
			return mxtcl_object_parse_error(
					interp, objv[2], "list", fname );

		tcl_status = Tcl_ListObjGetElements( interp, top_row_element,
					&list_length, &list_elements );

		if ( tcl_status != TCL_OK )
			return mxtcl_object_parse_error(
					interp, objv[2], "list", fname );

		/* Copy the preset_count_region boundaries in the list
		 * to the preset_count_region array.
		 */

		tcl_status = Tcl_GetLongFromObj( interp,
				list_elements[0], &lower_boundary );

		if ( tcl_status != TCL_OK )
			return mxtcl_object_parse_error(
				interp, list_elements[0], "long", fname );

		tcl_status = Tcl_GetLongFromObj( interp,
				list_elements[1], &upper_boundary );

		if ( tcl_status != TCL_OK )
			return mxtcl_object_parse_error(
				interp, list_elements[1], "long", fname );

		roi_array[i][0] = (unsigned long) lower_boundary;
		roi_array[i][1] = (unsigned long) upper_boundary;
	}

	/* Set all the ROI boundaries. */

	mx_status = mx_mca_set_roi_array( record, current_num_rois, roi_array );

	mx_free_array( roi_array, 2, dimension_array, element_size_array );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );
	
	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_roi_integrals_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_get_roi_integrals_cmd()";

	Tcl_Obj *resultPtr, *temp_object;
	MX_RECORD *record;
	MX_MCA *mca;
	unsigned long i, current_num_rois;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	/* Find out how many ROIs are currently in use. */

	mx_status = mx_mca_get_pointers( record, &mca, NULL, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	current_num_rois = mca->current_num_rois;

	/* Get the ROI integral values. */

	mx_status = mx_mca_get_roi_integral_array( record,
						current_num_rois, NULL );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	/* Create a new list to hold the returned ROI integral values. */

	temp_object = Tcl_NewObj();

	resultPtr = Tcl_GetObjResult( interp );

	Tcl_SetListObj( resultPtr, 0, NULL );

	/* Insert the ROI integral values into the list. */

	for ( i = 0; i < current_num_rois; i++ ) {

		Tcl_SetLongObj( temp_object, (mca->roi_integral_array)[i] );

		Tcl_ListObjAppendElement( interp, resultPtr, temp_object );
	}

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_num_channels_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	unsigned long num_channels;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_get_num_channels( record, &num_channels );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetLongObj( resultPtr, (long) num_channels );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_set_num_channels_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_set_num_channels_cmd()";

	MX_RECORD *record;
	int tcl_status;
	long num_channels;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &num_channels );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "long", fname );

	mx_status = mx_mca_set_num_channels( record,
					(unsigned long) num_channels );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_channel_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_get_channel_cmd()";

	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	int tcl_status;
	long channel_number;
	unsigned long channel_value;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &channel_number );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "long", fname );

	mx_status = mx_mca_get_channel( record, channel_number, &channel_value);

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetLongObj( resultPtr, (long) channel_value );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_real_time_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	double real_time;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_get_real_time( record, &real_time );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetDoubleObj( resultPtr, real_time );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_live_time_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	double live_time;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_get_live_time( record, &live_time );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetDoubleObj( resultPtr, live_time );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_counts_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	unsigned long counts;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_mca_get_counts( record, &counts );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetLongObj( resultPtr, (long) counts );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_soft_roi_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_get_soft_roi_cmd()";

	Tcl_Obj *resultPtr, *temp_object;
	MX_RECORD *record;
	int tcl_status;
	long soft_roi_number;
	unsigned long soft_roi[2];
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &soft_roi_number );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "long", fname );

	mx_status = mx_mca_get_soft_roi( record,
			(unsigned long) soft_roi_number, soft_roi );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	/* The region of interest boundaries are returned
	 * as a 2-element list.
	 */

	temp_object = Tcl_NewObj();

	resultPtr = Tcl_GetObjResult( interp );

	Tcl_SetListObj( resultPtr, 0, NULL );

	/* Insert the region of interest boundaries into the list. */

	Tcl_SetLongObj( temp_object, soft_roi[0] );
	Tcl_ListObjAppendElement( interp, resultPtr, temp_object );

	Tcl_SetLongObj( temp_object, soft_roi[1] );
	Tcl_ListObjAppendElement( interp, resultPtr, temp_object );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_set_soft_roi_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_set_soft_roi_cmd()";

	MX_RECORD *record;
	int tcl_status;
	long soft_roi_number;
	unsigned long soft_roi[2];
	long lower_boundary, upper_boundary;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &soft_roi_number );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "long", fname );

	/* Copy the soft_roi boundaries to the soft_roi array. */

	tcl_status = Tcl_GetLongFromObj( interp, objv[3], &lower_boundary );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, objv[3], "long", fname );

	tcl_status = Tcl_GetLongFromObj( interp, objv[4], &upper_boundary );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, objv[4], "long", fname );

	soft_roi[0] = (unsigned long) lower_boundary;
	soft_roi[1] = (unsigned long) upper_boundary;

	mx_status = mx_mca_set_soft_roi( record,
			(unsigned long) soft_roi_number, soft_roi );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_mca_get_soft_integral_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_mca_get_soft_integral_cmd()";

	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	int tcl_status;
	long soft_roi_number;
	unsigned long soft_roi_integral;
	mx_status_type mx_status;

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &soft_roi_number );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
					interp, objv[2], "long", fname );

	mx_status = mx_mca_get_soft_roi_integral( record,
						(unsigned long) soft_roi_number,
						&soft_roi_integral );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetLongObj( resultPtr, (long) soft_roi_integral );

	return TCL_OK;
}

