/*
 * Name:    mt_output.c
 *
 * Purpose: Tcl interface to generic MX output functions.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <tcl.h>

#include "mx_util.h"
#include "mx_record.h"
#include "mx_handle.h"
#include "mx_tcl.h"
#include "mx_tcl_private.h"

/*-------------------------------------------------------------------------*/

static Tcl_Interp *Mx_info_output_interp = NULL;
static Tcl_DString Mx_info_output_function_name;

static void Mx_info_output_handler( char * );

MX_EXPORT int
Mx_info_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "info";

	char *string_ptr;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: info message" );
		return TCL_ERROR;
	}

        string_ptr = Tcl_GetString( objv[1] );

        if ( string_ptr == NULL ) {
                mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
                        "The pointer returned by Tcl_GetString() is NULL." );
		return mxtcl_error( interp, mx_status );
        }

	mx_info( string_ptr );

	return TCL_OK;
}

MX_EXPORT int
Mx_set_info_output_function_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "set_info_output_function";

	char *string_ptr;
	int length;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: set_info_output_function function" );
		return TCL_ERROR;
	}

	string_ptr = Tcl_GetStringFromObj( objv[1], &length );

	if ( string_ptr == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
		"The pointer returned by Tcl_GetStringFromObj() is NULL." );
		return mxtcl_error( interp, mx_status );
	}

	/* Delete the old contents of the DString. */

	Tcl_DStringFree( &Mx_info_output_function_name );

	/* If the argument passed is a zero length string, then change
	 * the info output handler back to the default handler and exit.
	 */

	if ( length <= 0 ) {
		mx_set_info_output_function( NULL );

		Mx_info_output_interp = NULL;

		return TCL_OK;
	}

	/* Save the supplied interpreter and function name. */

	Mx_info_output_interp = interp;

	Tcl_DStringInit( &Mx_info_output_function_name );

	Tcl_DStringAppendElement( &Mx_info_output_function_name, string_ptr );

	/* Install an MX info output handler. */

	mx_set_info_output_function( Mx_info_output_handler );

	return TCL_OK;
}

static void
Mx_info_output_handler( char *message )
{
	static const char fname[] = "Mx_info_output_handler()";

	Tcl_DString command;
	int tcl_status;

	Tcl_DStringInit( &command );

	Tcl_DStringAppendElement( &command,
		Tcl_DStringValue( &Mx_info_output_function_name ) );

	Tcl_DStringAppendElement( &command, message );

	tcl_status = Tcl_EvalEx( Mx_info_output_interp,
				Tcl_DStringValue( &command ), -1, 0 );

	if ( tcl_status != TCL_OK ) {
		fprintf( stderr,
		"%s: Tcl_EvalEx() status = %d for command '%s'\n",
			fname, tcl_status, Tcl_DStringValue( &command ) );
	}

	Tcl_DStringFree( &command );

	return;
}

/*-------------------------------------------------------------------------*/

static Tcl_Interp *Mx_warning_output_interp = NULL;
static Tcl_DString Mx_warning_output_function_name;

static void Mx_warning_output_handler( char * );

MX_EXPORT int
Mx_warning_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "warning";

	char *string_ptr;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: warning message" );
		return TCL_ERROR;
	}

        string_ptr = Tcl_GetString( objv[1] );

        if ( string_ptr == NULL ) {
                mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
                        "The pointer returned by Tcl_GetString() is NULL." );
		return mxtcl_error( interp, mx_status );
        }

	mx_warning( string_ptr );

	return TCL_OK;
}

MX_EXPORT int
Mx_set_warning_output_function_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "set_warning_output_function";

	char *string_ptr;
	int length;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: set_warning_output_function function" );
		return TCL_ERROR;
	}

	string_ptr = Tcl_GetStringFromObj( objv[1], &length );

	if ( string_ptr == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
		"The pointer returned by Tcl_GetStringFromObj() is NULL." );
		return mxtcl_error( interp, mx_status );
	}

	/* Delete the old contents of the DString. */

	Tcl_DStringFree( &Mx_warning_output_function_name );

	/* If the argument passed is a zero length string, then change
	 * the warning output handler back to the default handler and exit.
	 */

	if ( length <= 0 ) {
		mx_set_warning_output_function( NULL );

		Mx_warning_output_interp = NULL;

		return TCL_OK;
	}

	/* Save the supplied interpreter and function name. */

	Mx_warning_output_interp = interp;

	Tcl_DStringInit( &Mx_warning_output_function_name );

	Tcl_DStringAppendElement( &Mx_warning_output_function_name,
					string_ptr );

	/* Install an MX warning output handler. */

	mx_set_warning_output_function( Mx_warning_output_handler );

	return TCL_OK;
}

static void
Mx_warning_output_handler( char *message )
{
	static const char fname[] = "Mx_warning_output_handler()";

	Tcl_DString command;
	int tcl_status;

	Tcl_DStringInit( &command );

	Tcl_DStringAppendElement( &command,
		Tcl_DStringValue( &Mx_warning_output_function_name ) );

	Tcl_DStringAppendElement( &command, message );

	tcl_status = Tcl_EvalEx( Mx_warning_output_interp,
				Tcl_DStringValue( &command ), -1, 0 );

	if ( tcl_status != TCL_OK ) {
		fprintf( stderr,
		"%s: Tcl_EvalEx() status = %d for command '%s'\n",
			fname, tcl_status, Tcl_DStringValue( &command ) );
	}

	Tcl_DStringFree( &command );

	return;
}

/*-------------------------------------------------------------------------*/

static Tcl_Interp *Mx_error_output_interp = NULL;
static Tcl_DString Mx_error_output_function_name;

static void Mx_error_output_handler( char * );

#if 0

MX_EXPORT int
Mx_error_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "error";

	char *string_ptr;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: error message" );
		return TCL_ERROR;
	}

        string_ptr = Tcl_GetString( objv[1] );

        if ( string_ptr == NULL ) {
                mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
                        "The pointer returned by Tcl_GetString() is NULL." );
		return mxtcl_error( interp, mx_status );
        }

	mx_error( string_ptr );

	return TCL_OK;
}

#endif

MX_EXPORT int
Mx_set_error_output_function_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "set_error_output_function";

	char *string_ptr;
	int length;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: set_error_output_function function" );
		return TCL_ERROR;
	}

	string_ptr = Tcl_GetStringFromObj( objv[1], &length );

	if ( string_ptr == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
		"The pointer returned by Tcl_GetStringFromObj() is NULL." );
		return mxtcl_error( interp, mx_status );
	}

	/* Delete the old contents of the DString. */

	Tcl_DStringFree( &Mx_error_output_function_name );

	/* If the argument passed is a zero length string, then change
	 * the error output handler back to the default handler and exit.
	 */

	if ( length <= 0 ) {
		mx_set_error_output_function( NULL );

		Mx_error_output_interp = NULL;

		return TCL_OK;
	}

	/* Save the supplied interpreter and function name. */

	Mx_error_output_interp = interp;

	Tcl_DStringInit( &Mx_error_output_function_name );

	Tcl_DStringAppendElement( &Mx_error_output_function_name, string_ptr );

	/* Install an MX error output handler. */

	mx_set_error_output_function( Mx_error_output_handler );

	return TCL_OK;
}

static void
Mx_error_output_handler( char *message )
{
	static const char fname[] = "Mx_error_output_handler()";

	Tcl_DString command;
	int tcl_status;

	Tcl_DStringInit( &command );

	Tcl_DStringAppendElement( &command,
		Tcl_DStringValue( &Mx_error_output_function_name ) );

	Tcl_DStringAppendElement( &command, message );

	tcl_status = Tcl_EvalEx( Mx_error_output_interp,
				Tcl_DStringValue( &command ), -1, 0 );

	if ( tcl_status != TCL_OK ) {
		fprintf( stderr,
		"%s: Tcl_EvalEx() status = %d for command '%s'\n",
			fname, tcl_status, Tcl_DStringValue( &command ) );
	}

	Tcl_DStringFree( &command );

	return;
}

/*-------------------------------------------------------------------------*/

static Tcl_Interp *Mx_debug_output_interp = NULL;
static Tcl_DString Mx_debug_output_function_name;

static void Mx_debug_output_handler( char * );

MX_EXPORT int
Mx_debug_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "debug";

	char *string_ptr;
	int tcl_status, current_debug_level, debug_level;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: debug debug_level message" );
		return TCL_ERROR;
	}

	tcl_status = Tcl_GetIntFromObj( Mx_debug_output_interp,
						objv[1], &debug_level );

	if ( tcl_status != TCL_OK )
		return tcl_status;

	current_debug_level = mx_get_debug_level();

	if ( debug_level > current_debug_level )
		return TCL_OK;

        string_ptr = Tcl_GetString( objv[2] );

        if ( string_ptr == NULL ) {
                mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
                        "The pointer returned by Tcl_GetString() is NULL." );
		return mxtcl_error( interp, mx_status );
        }

	MX_DEBUG( debug_level, ( "%s", string_ptr ) );

	return TCL_OK;
}

MX_EXPORT int
Mx_set_debug_output_function_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "set_debug_output_function";

	char *string_ptr;
	int length;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: set_debug_output_function function" );
		return TCL_ERROR;
	}

	string_ptr = Tcl_GetStringFromObj( objv[1], &length );

	if ( string_ptr == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
		"The pointer returned by Tcl_GetStringFromObj() is NULL." );
		return mxtcl_error( interp, mx_status );
	}

	/* Delete the old contents of the DString. */

	Tcl_DStringFree( &Mx_debug_output_function_name );

	/* If the argument passed is a zero length string, then change
	 * the debug output handler back to the default handler and exit.
	 */

	if ( length <= 0 ) {
		mx_set_debug_output_function( NULL );

		Mx_debug_output_interp = NULL;

		return TCL_OK;
	}

	/* Save the supplied interpreter and function name. */

	Mx_debug_output_interp = interp;

	Tcl_DStringInit( &Mx_debug_output_function_name );

	Tcl_DStringAppendElement( &Mx_debug_output_function_name, string_ptr );

	/* Install an MX debug output handler. */

	mx_set_debug_output_function( Mx_debug_output_handler );

	return TCL_OK;
}

static void
Mx_debug_output_handler( char *message )
{
	static const char fname[] = "Mx_debug_output_handler()";

	Tcl_DString command;
	int tcl_status;

	Tcl_DStringInit( &command );

	Tcl_DStringAppendElement( &command,
		Tcl_DStringValue( &Mx_debug_output_function_name ) );

	Tcl_DStringAppendElement( &command, message );

	tcl_status = Tcl_EvalEx( Mx_debug_output_interp,
				Tcl_DStringValue( &command ), -1, 0 );

	if ( tcl_status != TCL_OK ) {
		fprintf( stderr,
		"%s: Tcl_EvalEx() status = %d for command '%s'\n",
			fname, tcl_status, Tcl_DStringValue( &command ) );
	}

	Tcl_DStringFree( &command );

	return;
}

/*-------------------------------------------------------------------------*/

static Tcl_Interp *Mx_info_dialog_interp = NULL;
static Tcl_DString Mx_info_dialog_function_name;

static void Mx_info_dialog_handler( char *, char *, char * );

MX_EXPORT int
Mx_info_dialog_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "info_dialog";

	char *text_prompt, *gui_prompt, *button_label;
	mx_status_type mx_status;

	if ( objc != 4 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: info_dialog text_prompt, gui_prompt, button_label" );
		return TCL_ERROR;
	}

	text_prompt = Tcl_GetStringFromObj( objv[1], NULL );

	if ( text_prompt == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
	"The text_prompt pointer returned by Tcl_GetStringFromObj() is NULL." );
		return mxtcl_error( interp, mx_status );
	}

	gui_prompt = Tcl_GetStringFromObj( objv[1], NULL );

	if ( gui_prompt == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
	"The gui_prompt pointer returned by Tcl_GetStringFromObj() is NULL." );
		return mxtcl_error( interp, mx_status );
	}

	button_label = Tcl_GetStringFromObj( objv[1], NULL );

	if ( button_label == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
"The button_label pointer returned by Tcl_GetStringFromObj() is NULL." );
		return mxtcl_error( interp, mx_status );
	}

	mx_info_dialog( text_prompt, gui_prompt, button_label );

	return TCL_OK;
}

MX_EXPORT int
Mx_set_info_dialog_function_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "set_info_dialog_function";

	char *string_ptr;
	int length;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: set_info_dialog_function function" );
		return TCL_ERROR;
	}

	string_ptr = Tcl_GetStringFromObj( objv[1], &length );

	if ( string_ptr == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
		"The pointer returned by Tcl_GetStringFromObj() is NULL." );
		return mxtcl_error( interp, mx_status );
	}

	/* Delete the old contents of the DString. */

	Tcl_DStringFree( &Mx_info_dialog_function_name );

	/* If the argument passed is a zero length string, then change
	 * the info dialog handler back to the default handler and exit.
	 */

	if ( length <= 0 ) {
		mx_set_info_dialog_function( NULL );

		Mx_info_dialog_interp = NULL;

		return TCL_OK;
	}

	/* Save the supplied interpreter and function name. */

	Mx_info_dialog_interp = interp;

	Tcl_DStringInit( &Mx_info_dialog_function_name );

	Tcl_DStringAppendElement( &Mx_info_dialog_function_name, string_ptr );

	/* Install an MX info dialog handler. */

	mx_set_info_dialog_function( Mx_info_dialog_handler );

	return TCL_OK;
}

static void
Mx_info_dialog_handler( char *text_prompt,
			char *gui_prompt,
			char *button_label )
{
	static const char fname[] = "Mx_info_dialog_handler()";

	Tcl_DString command;
	int tcl_status;

	Tcl_DStringInit( &command );

	Tcl_DStringAppendElement( &command,
		Tcl_DStringValue( &Mx_info_dialog_function_name ) );

	Tcl_DStringAppendElement( &command, text_prompt );

	Tcl_DStringAppendElement( &command, gui_prompt );

	Tcl_DStringAppendElement( &command, button_label );

	tcl_status = Tcl_EvalEx( Mx_info_dialog_interp,
				Tcl_DStringValue( &command ), -1, 0 );

	if ( tcl_status != TCL_OK ) {
		fprintf( stderr,
		"%s: Tcl_EvalEx() status = %d for command '%s'\n",
			fname, tcl_status, Tcl_DStringValue( &command ) );
	}

	Tcl_DStringFree( &command );

	return;
}

/*-------------------------------------------------------------------------*/

MX_EXPORT void
Mx_output_dstring_init( void )
{
	Tcl_DStringInit( &Mx_info_output_function_name );
	Tcl_DStringInit( &Mx_warning_output_function_name );
	Tcl_DStringInit( &Mx_error_output_function_name );
	Tcl_DStringInit( &Mx_debug_output_function_name );
	Tcl_DStringInit( &Mx_info_dialog_function_name );
	return;
}

