/*
 * Name:    mt_user_interrupt.c
 *
 * Purpose: Tcl interface to generic MX user interrupt functions.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2002, 2004, 2006 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <tcl.h>

#include "mx_util.h"
#include "mx_record.h"
#include "mx_scan.h"
#include "mx_handle.h"
#include "mx_key.h"
#include "mx_tcl.h"
#include "mx_tcl_private.h"

/*-------------------------------------------------------------------------*/

MX_EXPORT int
Mx_getch_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	int character;

	if ( objc != 1 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: getch" );
		return TCL_ERROR;
	}

	character = mx_getch();

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, character );

	return TCL_OK;
}

MX_EXPORT int
Mx_kbhit_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	int return_value;

	if ( objc != 1 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: kbhit" );
		return TCL_ERROR;
	}

	return_value = mx_kbhit();

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, return_value );

	return TCL_OK;
}

/*-------------------------------------------------------------------------*/

static Tcl_Interp *Mx_user_interrupt_interp = NULL;
static Tcl_DString Mx_user_interrupt_function_name;

static int Mx_user_interrupt_handler( void );

MX_EXPORT void
Mx_user_interrupt_dstring_init( void )
{
	Tcl_DStringInit( &Mx_user_interrupt_function_name );
	return;
}

MX_EXPORT int
Mx_user_requested_interrupt_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	int user_interrupt;

	if ( objc != 1 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: user_requested_interrupt" );
		return TCL_ERROR;
	}

	user_interrupt = mx_user_requested_interrupt();

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, user_interrupt );

	return TCL_OK;
}

MX_EXPORT int
Mx_set_user_interrupt_function_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "set_user_interrupt_function";

	char *string_ptr;
	int length;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: set_user_interrupt_function function" );
		return TCL_ERROR;
	}

	string_ptr = Tcl_GetStringFromObj( objv[1], &length );

	if ( string_ptr == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
		"The pointer returned by Tcl_GetStringFromObj() is NULL." );
		return mxtcl_error( interp, mx_status );
	}

	/* Delete the old contents of the DString. */

	Tcl_DStringFree( &Mx_user_interrupt_function_name );

	/* If the argument passed is a zero length string, then change
	 * the user interrupt handler back to the default handler and exit.
	 */

	if ( length <= 0 ) {
		mx_set_user_interrupt_function( NULL );

		Mx_user_interrupt_interp = NULL;

		return TCL_OK;
	}

	/* Save the supplied interpreter and function name. */

	Mx_user_interrupt_interp = interp;

	Tcl_DStringInit( &Mx_user_interrupt_function_name );

	Tcl_DStringAppendElement( &Mx_user_interrupt_function_name,
					string_ptr );

	/* Install an MX user interrupt handler. */

	mx_set_user_interrupt_function( Mx_user_interrupt_handler );

	return TCL_OK;
}

static int
Mx_user_interrupt_handler( void )
{
	static const char fname[] = "Mx_user_interrupt_handler()";

	Tcl_Obj *resultPtr;
	const char *string_result;
	char *user_interrupt_function_name;
	int tcl_status, user_interrupt;

	user_interrupt_function_name = Tcl_DStringValue(
					&Mx_user_interrupt_function_name ),

	/* Run the user interrupt handler. */

	tcl_status = Tcl_EvalEx( Mx_user_interrupt_interp,
			user_interrupt_function_name, -1, 0 );

	/* Did the interrupt handler abort with an error? */

	switch( tcl_status ) {
	case TCL_BREAK:
		(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
	"Unexpected 'break' statement seen in user interrupt handler '%s'.",
				user_interrupt_function_name );

		return MXF_USER_INT_ERROR;
		break;
	case TCL_CONTINUE:
		(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
	"Unexpected 'continue' statement seen in user interrupt handler '%s'.",
				user_interrupt_function_name );

		return MXF_USER_INT_ERROR;
		break;
	case TCL_ERROR:
		string_result = Tcl_GetStringResult(
						Mx_user_interrupt_interp );
		(void) mx_error( MXE_FUNCTION_FAILED, fname,
	"User interrupt handler '%s' terminated with error '%s'.",
				user_interrupt_function_name, string_result );
		return MXF_USER_INT_ERROR;
		break;
	case TCL_OK:
	case TCL_RETURN:
		break;
	}

	/* Parse the result from the user interrupt handler. */

	resultPtr = Tcl_GetObjResult( Mx_user_interrupt_interp );

	tcl_status = Tcl_GetIntFromObj( Mx_user_interrupt_interp,
				resultPtr, &user_interrupt );

	switch( tcl_status ) {
	case TCL_OK:
	case TCL_RETURN:
		break;

	case TCL_BREAK:
		(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
	"Unexpected 'break' statement seen while parsing the result from "
	"user interrupt handler '%s'.", user_interrupt_function_name );

		return MXF_USER_INT_ERROR;
		break;
	case TCL_CONTINUE:
		(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
	"Unexpected 'continue' statement seen while parsing the result from "
	"user interrupt handler '%s'.", user_interrupt_function_name );

		return MXF_USER_INT_ERROR;
		break;
	case TCL_ERROR:
		string_result = Tcl_GetStringResult(
						Mx_user_interrupt_interp );
		(void) mx_error( MXE_FUNCTION_FAILED, fname,
	"Unexpected error '%s' occurred while parsing the result from "
	"user interrupt handler '%s'.", string_result,
				user_interrupt_function_name );
		return MXF_USER_INT_ERROR;
		break;
	}

	return user_interrupt;
}

/*-------------------------------------------------------------------------*/

static Tcl_Interp *Mx_scan_pause_request_interp = NULL;
static Tcl_DString Mx_scan_pause_request_handler_name;

static mx_status_type Mx_scan_pause_request_handler( MX_SCAN * );

MX_EXPORT void
Mx_scan_pause_request_dstring_init( void )
{
	Tcl_DStringInit( &Mx_scan_pause_request_handler_name );
	return;
}

MX_EXPORT int
Mx_set_scan_pause_request_handler_cmd( ClientData clientData,
		Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[] )
{
	static const char cname[] = "set_scan_pause_request_handler";

	char *string_ptr;
	int length;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: set_scan_pause_request_handler function" );
		return TCL_ERROR;
	}

	string_ptr = Tcl_GetStringFromObj( objv[1], &length );

	if ( string_ptr == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, cname,
		"The pointer returned by Tcl_GetStringFromObj() is NULL." );
		return mxtcl_error( interp, mx_status );
	}

	/* Delete the old contents of the DString. */

	Tcl_DStringFree( &Mx_scan_pause_request_handler_name );

	/* If the argument passed is a zero length string, then change
	 * the scan pause request handler back to the default handler and exit.
	 */

	if ( length <= 0 ) {
		mx_set_scan_pause_request_handler( NULL );

		Mx_scan_pause_request_interp = NULL;

		return TCL_OK;
	}

	/* Save the supplied interpreter and function name. */

	Mx_scan_pause_request_interp = interp;

	Tcl_DStringInit( &Mx_scan_pause_request_handler_name );

	Tcl_DStringAppendElement( &Mx_scan_pause_request_handler_name,
					string_ptr );

	/* Install an MX scan pause request handler. */

	mx_set_scan_pause_request_handler( Mx_scan_pause_request_handler );

	return TCL_OK;
}

static mx_status_type
Mx_scan_pause_request_handler( MX_SCAN *scan )
{
	static const char fname[] = "Mx_scan_pause_request_handler()";

	Tcl_Obj *resultPtr;
	const char *string_result;
	MX_RECORD *scan_record;
	char *scan_pause_request_handler_name;
	int tcl_status, scan_pause_request_code;
	char scan_pause_request_command[200];

	/* Get the record corresponding to this scan. */

	if ( scan == (MX_SCAN *) NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
		"The MX_SCAN pointer passed was NULL." );
	}

	scan_record = scan->record;

	if ( scan_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
	    "The MX_RECORD pointer for the MX_SCAN pointer passed was NULL.");
	}

	/* Run the scan pause request handler. */

	scan_pause_request_handler_name = Tcl_DStringValue(
					&Mx_scan_pause_request_handler_name ),

	snprintf( scan_pause_request_command,
		sizeof(scan_pause_request_command),
		"%s [ Mx::object_name %s ]",
		scan_pause_request_handler_name,
		scan_record->name );

	tcl_status = Tcl_EvalEx( Mx_scan_pause_request_interp,
			scan_pause_request_command, -1, 0 );

	/* Did the pause request handler abort with an error? */

	switch( tcl_status ) {
	case TCL_BREAK:
		return mx_error( MXE_ILLEGAL_ARGUMENT, fname,
	"Unexpected 'break' statement seen in scan pause request handler '%s'.",
				scan_pause_request_handler_name );
		break;
	case TCL_CONTINUE:
		return mx_error( MXE_ILLEGAL_ARGUMENT, fname,
    "Unexpected 'continue' statement seen in scan pause request handler '%s'.",
				scan_pause_request_handler_name );
		break;
	case TCL_ERROR:
		string_result = Tcl_GetStringResult(
						Mx_scan_pause_request_interp );
		return mx_error( MXE_FUNCTION_FAILED, fname,
    "Scan pause request handler '%s' terminated with error '%s'.",
				scan_pause_request_handler_name, string_result);
		break;
	case TCL_OK:
	case TCL_RETURN:
		break;
	}

	/* Parse the result from the scan pause request handler. */

	resultPtr = Tcl_GetObjResult( Mx_scan_pause_request_interp );

	tcl_status = Tcl_GetIntFromObj( Mx_scan_pause_request_interp,
				resultPtr, &scan_pause_request_code );

	switch( tcl_status ) {
	case TCL_OK:
	case TCL_RETURN:
		break;

	case TCL_BREAK:
		return mx_error( MXE_ILLEGAL_ARGUMENT, fname,
	"Unexpected 'break' statement seen while parsing the result from "
	"scan pause request handler '%s'.", scan_pause_request_handler_name );
		break;
	case TCL_CONTINUE:
		return mx_error( MXE_ILLEGAL_ARGUMENT, fname,
	"Unexpected 'continue' statement seen while parsing the result from "
	"scan pause request handler '%s'.", scan_pause_request_handler_name );
		break;
	case TCL_ERROR:
		string_result = Tcl_GetStringResult(
						Mx_scan_pause_request_interp );
		return mx_error( MXE_FUNCTION_FAILED, fname,
	"Unexpected error '%s' occurred while parsing the result from "
	"scan pause request handler '%s'.", string_result,
				scan_pause_request_handler_name );
		break;
	}

	switch( scan_pause_request_code ) {
	case MXE_SUCCESS:
		return MX_SUCCESSFUL_RESULT;
		break;
	case MXE_STOP_REQUESTED:
		return mx_error( MXE_STOP_REQUESTED, fname,
	"Scan motors will be stopped at the end of the current step." );
		break;
	case MXE_INTERRUPTED:
		return mx_error( MXE_INTERRUPTED, fname, "Scan aborted." );
		break;
	default:
		return mx_error( MXE_ILLEGAL_ARGUMENT, fname,
		"Unexpected MX status code %d returned by "
		"scan pause request handler '%s'",
			scan_pause_request_code,
			scan_pause_request_handler_name );
		break;
	}
}

